#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>


#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "buehlmann.h"
#include "udcfparse.h"
#include "plot.h"


//#define DEBUG 1

#ifdef DEBUG
#define DPRINTF printf
#else
#define DPRINTF if(0) printf
#endif


extern dive_data my_dive;       // buehlmann dive data
struct udcf_profile_struct* udcf;
extern GtkWidget *main_window;

/* store of the compart model treeview */
extern GtkListStore *store;
extern GtkTreeIter iter;


// FIXME: need to do something about acessing this treeview thing 
static  GtkWidget *treeview;

/* backing pixmap */
static GdkPixmap *pixmap = NULL;

/* file name */
const char *selected_filename;

/* save or load */
gboolean file_load_mode = TRUE;

/* file loaded? */
gboolean file_loaded    = FALSE;


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  GtkWidget *about_text;
  GtkWidget *about_dialog;

  GtkTextBuffer *about_text_buffer;
  GtkTextIter txtstart, txtend;
 
  char str[256];

  about_dialog = create_about_dialog ();
  gtk_widget_show (about_dialog);


  about_text = lookup_widget(GTK_WIDGET(about_dialog), "about_text");

  about_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (about_text));
 
  gtk_text_buffer_get_bounds (about_text_buffer, &txtstart, &txtend); 
  gtk_text_buffer_delete (about_text_buffer, &txtstart, &txtend);
  sprintf(str,"\ngtkdive %s\n\nThis is a Buehlmann ZH-L16 model diving simulation\nby Matthias Wientapper\n(m.wientapper@gmx.de)\n\nDONT USE THIS SOFTWARE TO PLAN A DIVE!", VERSION);

  gtk_text_buffer_insert_at_cursor (about_text_buffer, str, -1);

}

void
on_button_about_ok_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


/* display/update  computed buehlmann values */
void update_values(GtkWidget       *button)
{
  GtkWidget *entry_pamb;
  GtkWidget *entry_time;
  GtkWidget *entry_ppn2;
  GtkWidget *entry_ppo2;
  GtkWidget *entry_ndt;
  GtkWidget *entry_stop;
  GtkWidget *entry_defc;
  GtkWidget *entry_depth;
  GtkWidget *entry_samples;

  GtkWidget *statusbar;
 
  GtkWidget *textview;
  GtkWidget *pbar;

//   GdkFont *fixed_font, *bold_font;
//   GtkTextBuffer *buffer;
//   GtkTextIter txtstart, txtend;
//   PangoFontDescription *font_desc;
   

 
  GtkTreeIter iter;


  gchar t_12[10];
  gchar pamb_tol[10];
  gchar pig_tol[10];
  gchar te[10];

  gchar entry_text[80];

  gchar *mylist[7];   // list of pointers to strings
  gchar text[7][10];  // seven strings with 10 chars
 
  gfloat perc_sat_amb[16];
  gfloat perc_sat_amb_tol[16];

  gint x,y;
  gfloat stop_depth;

  guint context_id;

  enum
    {
      COLUMN_NUMBER,
      COLUMN_T_12,
      COLUMN_PAMB_TOL,
      COLUMN_PIG_TOL,
      COLUMN_TE,
      NUM_COLUMNS
    };

  entry_pamb    = lookup_widget(GTK_WIDGET(button), "entry_pamb");
  entry_ppn2    = lookup_widget(GTK_WIDGET(button), "entry_ppn2");
  entry_ppo2    = lookup_widget(GTK_WIDGET(button), "entry_ppo2");
  entry_ndt     = lookup_widget(GTK_WIDGET(button), "entry_ndt");
  entry_time    = lookup_widget(GTK_WIDGET(button), "entry_time");
  entry_stop    = lookup_widget(GTK_WIDGET(button), "entry_stop");
  entry_defc    = lookup_widget(GTK_WIDGET(button), "entry_defc");
  statusbar     = lookup_widget(GTK_WIDGET(button), "statusbar1");
  entry_depth   = lookup_widget(GTK_WIDGET(button), "entry_depth");
  entry_samples = lookup_widget(GTK_WIDGET(button), "entry_samples");

  context_id = gtk_statusbar_get_context_id( GTK_STATUSBAR (statusbar), 
					     "gtkdive statusbar");
 
  sprintf(entry_text,"%.2f", my_dive.p_amb);
  gtk_entry_set_text(GTK_ENTRY(entry_pamb), entry_text);

  sprintf(entry_text,"%.1f", my_dive.time / 60);
  gtk_entry_set_text(GTK_ENTRY(entry_time), entry_text);

  sprintf(entry_text,"%.2f", my_dive.pp_n2);
  gtk_entry_set_text(GTK_ENTRY(entry_ppn2), entry_text);

  sprintf(entry_text,"%.2f", my_dive.pp_o2);
  gtk_entry_set_text(GTK_ENTRY(entry_ppo2), entry_text);

  sprintf(entry_text,"%.2f", my_dive.te_min);
  gtk_entry_set_text(GTK_ENTRY(entry_ndt), entry_text);

  sprintf(entry_text,"%.2f [%.2f]", my_dive.depth, my_dive.depth_max);
  gtk_entry_set_text(GTK_ENTRY(entry_depth), entry_text);

  sprintf(entry_text,"%d [%d]", my_dive.num_samples-1, my_dive.num_file_samples);
  gtk_entry_set_text(GTK_ENTRY(entry_samples), entry_text);

  /* print compartment parameters */
	  

  /* putting data to list */
  gtk_list_store_clear (store);
  for (y=0; y<16; y++){
    
    sprintf(t_12,"%3.0f", my_dive.compart[y][C_T1_2]);
    sprintf(pamb_tol,"%2.2f", (my_dive.compart[y][C_PAMBTOL]>0 ? my_dive.compart[y][C_PAMBTOL] : 0.0));
    sprintf(pig_tol,"%2.2f", my_dive.compart[y][C_PIG_TOL]);
    sprintf(te,"%2.2f", my_dive.compart[y][C_TE]);

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter,
 			COLUMN_NUMBER, y+1,
 			COLUMN_T_12, t_12,
 			COLUMN_PAMB_TOL, pamb_tol,
 			COLUMN_PIG_TOL, pig_tol,
 			COLUMN_TE, te,
 			-1);
  }
 
  /* next stop */
  //  stop_depth = (my_dive.pp_amb_tol_max - 1.0 ) * 10.0;
  if (my_dive.depth_tol_min >= 0){
    sprintf(entry_text,"%.2f", my_dive.depth_tol_min);
    gtk_entry_set_text(GTK_ENTRY(entry_stop), entry_text);
  }

  /* defining compartment */
  sprintf(entry_text,"%d", my_dive.def_cmpt);
  gtk_entry_set_text(GTK_ENTRY(entry_defc), entry_text);

  /* status bar */
  if (my_dive.deco){
    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
    gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, "Decompression stops mandatory!");
  } else {
    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
  }

  /* progress bars */
  for (y=0; y<16; y++){
    perc_sat_amb[y] = my_dive.compart[y][C_PIG] / my_dive.pp_n2;
    if(perc_sat_amb[y] > 1.0)
      perc_sat_amb[y] = 1.0;
    
    perc_sat_amb_tol[y] = my_dive.compart[y][C_PIG] / my_dive.compart[y][C_PIG_TOL];
    if(perc_sat_amb_tol[y] > 1.0)
      perc_sat_amb_tol[y] = 1.0;
  }

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar1");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[0] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar2");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[0] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar3");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[1] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar4");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[1] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar5");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[2] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar6");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[2] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar7");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[3] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar8");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[3] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar9");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[4] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar10");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[4] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar11");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[5] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar12");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[5] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar13");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[6] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar14");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[6] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar15");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[7] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar16");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[7] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar17");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[8] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar18");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[8] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar19");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[9] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar20");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[9] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar21");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[10] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar22");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[10] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar23");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[11] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar24");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[11] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar25");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[12] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar26");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[12] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar27");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[13] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar28");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[13] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar29");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[14] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar30");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[14] );

  pbar = lookup_widget(GTK_WIDGET(button), "progressbar31");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb[15] );
  pbar = lookup_widget(GTK_WIDGET(button), "progressbar32");
  gtk_progress_bar_update( GTK_PROGRESS_BAR(pbar), perc_sat_amb_tol[15] );

  plot_dive(pixmap, button);
}


void
on_button_dive_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  int i;
  GtkWidget *spin;
  spin         = lookup_widget(GTK_WIDGET(button), "spinbutton_depth");

  if(!file_loaded){
    calculate_tissues(&my_dive);
    update_values(GTK_WIDGET(button));
  } else {
    /* wrap if end of dive was reached */
    if (my_dive.num_samples > my_dive.num_file_samples){

     my_dive.deco         = 0;
     my_dive.depth        = 0.0;
     my_dive.depth_old    = 0.0;
     my_dive.time         = 0.0;
     my_dive.p_amb_air    = 1.001;
     my_dive.te_min       = 999.0;
     my_dive.depth_max    = 0.0;
     my_dive.depth_tol_min = 0.0;
     my_dive.depth_samples[0] = 0.0; /* dive starts always with depth=0.0! */
     my_dive.num_samples = 1;        /* for that reason we have on offset of 1 */
     
     calculate_pressure(&my_dive);
     /* initialize compart model values */
     for(i=0;i<16;i++){
       my_dive.compart[i][C_PAMBTOL]=0;
       my_dive.compart[i][C_PIG_TOL]=0;
       my_dive.compart[i][C_PIG]=0;
       my_dive.compart[i][C_TE]=0;
     }
     /* inert gas pressure in balance, t0=te in all tissues */
     for (i=0; i<16; i++){
       my_dive.compart[i][C_PIG] = my_dive.pp_n2;
     }


    }
    my_dive.depth = my_dive.depth_samples[my_dive.num_samples];
    //   gtk_spin_button_set_value (GTK_SPIN_BUTTON(spin), my_dive.depth);

    calculate_tissues(&my_dive);
    update_values(GTK_WIDGET(button));
  }

}



void
on_button_clear_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *spin;
  GtkWidget *drawing_area;


 
  drawing_area = lookup_widget(GTK_WIDGET(button), "drawingarea1");
  spin         = lookup_widget(GTK_WIDGET(button), "spinbutton_depth");
 
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(spin), 0.0);

  init_dive_data(&my_dive);

  spin   = lookup_widget(GTK_WIDGET(button), "spinbutton_c_o2");
  my_dive.perc_o2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
  spin   = lookup_widget(GTK_WIDGET(button), "spinbutton_delta_time");
  my_dive.delta_time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

 
  update_values(GTK_WIDGET(button));

  gdk_draw_rectangle (pixmap,
		      drawing_area->style->white_gc,
		      TRUE,
		      0, 0,
		      drawing_area->allocation.width,
		      drawing_area->allocation.height);

}


/* configure_event
 *
 * Create a new backing pixmap of the appropriate size
 * Of course, this is called whenever the window is
 * resized.  We have to free up things we allocated.
 */
gboolean
on_drawingarea1_configure_event        (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
  /* --- Free background if we created it --- */
  if (pixmap) {
    gdk_pixmap_unref (pixmap);
  }
    
  /* --- Create a new pixmap with new size --- */
  pixmap = gdk_pixmap_new (widget->window,
			   widget->allocation.width,
			   widget->allocation.height,
			   -1);
  plot_dive(pixmap, widget);
  return TRUE;
}

/*
 * expose_event
 *
 * When the window is exposed to the viewer or
 * the gdk_widget_draw routine is called, this
 * routine is called.  Copies the background pixmap
 * to the window.
 */
gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  /* --- Copy pixmap to the window --- */
  gdk_draw_pixmap (widget->window,
		   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		   pixmap,
		   event->area.x, event->area.y,
		   event->area.x, event->area.y,
		   event->area.width, event->area.height);
  return FALSE;
}

 

void
on_spinbutton_delta_time_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget *spin;
  spin   = lookup_widget(GTK_WIDGET(editable), "spinbutton_delta_time");
  my_dive.delta_time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
}


void
on_spinbutton_c_o2_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget *spin;
  spin   = lookup_widget(GTK_WIDGET(editable), "spinbutton_c_o2");
  my_dive.perc_o2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
  calculate_pressure(&my_dive);
  update_values(spin);
}


void
on_spinbutton_depth_changed            (GtkEditable     *editable,
                                        gpointer         user_data)

{
  GtkWidget *spin;

  spin   = lookup_widget(GTK_WIDGET(editable), "spinbutton_depth");

  my_dive.depth = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
  file_loaded = FALSE;
  calculate_pressure(&my_dive);
  update_values(spin);
}




void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *preferences;
 
  preferences = create_preferences_dialog ();
  gtk_widget_show (preferences);
}


void
on_prefcancelbutton_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_prefokbutton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *spin;
  gfloat altitude;

  spin = lookup_widget(GTK_WIDGET(button), "pref_altitude");
  altitude = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));

  my_dive.p_amb_air = (1-0.000126*altitude);

  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}



void
on_blend_calculate_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *blend_eanx_from;
  GtkWidget *blend_start_pressure;
  GtkWidget *blend_eanx_to;
  GtkWidget *blend_end_pressure;
  GtkWidget *blend_oxygen;
  GtkWidget *blend_topping;
  GtkWidget *blend_plan;

  GtkWidget *ead_depth;
  GtkWidget *ead_ead;
  GtkWidget *ead_eanx;

  GtkWidget *mod_eanx;
  GtkWidget *mod_mod;
  GtkWidget *mod_ppo2_max;

  GtkTextBuffer *buffer;
  GtkTextIter txtstart, txtend;
  PangoFontDescription *font_desc;

  gfloat eanx_from;
  gfloat start_pressure;
  gfloat eanx_to;
  gfloat end_pressure;
  gfloat oxygen;
  gfloat topping;

  gfloat eanx, mod, ppo2_max, ead;

  gchar str[128];


  blend_eanx_from      = lookup_widget(GTK_WIDGET(button), "blend_eanx_from");
  blend_start_pressure = lookup_widget(GTK_WIDGET(button), "blend_start_pressure");
  blend_eanx_to        = lookup_widget(GTK_WIDGET(button), "blend_eanx_to");
  blend_end_pressure   = lookup_widget(GTK_WIDGET(button), "blend_end_pressure");
  blend_oxygen         = lookup_widget(GTK_WIDGET(button), "blend_oxygen");
  blend_topping        = lookup_widget(GTK_WIDGET(button), "blend_topping");
  blend_plan           = lookup_widget(GTK_WIDGET(button), "blend_plan");

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (blend_plan));
  font_desc = pango_font_description_from_string ("Courier 10 Pitch");
  gtk_widget_modify_font (blend_plan, font_desc);
  pango_font_description_free (font_desc);
  gtk_text_buffer_get_bounds (buffer, &txtstart, &txtend); 
  gtk_text_buffer_delete (buffer, &txtstart, &txtend);

  eanx_from      = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(blend_eanx_from));
  start_pressure = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(blend_start_pressure));
  eanx_to        = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(blend_eanx_to));
  end_pressure   = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(blend_end_pressure));
  
  calculate_blend( eanx_from,
		   start_pressure,
		   eanx_to,
		   end_pressure,
		   &oxygen,
		   &topping);

 
  sprintf(str,"%.2f", topping);
  gtk_entry_set_text(GTK_ENTRY(blend_topping), str);

  sprintf(str,"%.2f", oxygen);
  gtk_entry_set_text(GTK_ENTRY(blend_oxygen), str);

  sprintf(str,"Add Oxygen to %.2f,\ntop with pure air to %.2f", start_pressure + oxygen, start_pressure + oxygen + topping);
  gtk_text_buffer_insert_at_cursor (buffer, str, -1);


  // calculate MOD for this mix
  mod_eanx     = lookup_widget(GTK_WIDGET(button), "mod_eanx");
  mod_mod      = lookup_widget(GTK_WIDGET(button), "mod_mod");
  mod_ppo2_max = lookup_widget(GTK_WIDGET(button), "mod_ppo2_max");
  
  ppo2_max = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(mod_ppo2_max));
  calculate_mod (eanx_to, ppo2_max, &mod);
  sprintf(str,"%.2f", mod);
  gtk_entry_set_text(GTK_ENTRY(mod_mod), str);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mod_eanx), eanx_to);

  // calculate EAD for this mix
  ead_depth      = lookup_widget(GTK_WIDGET(button), "ead_depth");
  ead_ead        = lookup_widget(GTK_WIDGET(button), "ead_ead");
  ead_eanx       = lookup_widget(GTK_WIDGET(button), "ead_eanx");

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ead_eanx), eanx_to);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ead_depth), mod);
  
  calculate_ead(eanx_to, mod, &ead);
  sprintf(str,"%.2f", ead);
  gtk_entry_set_text(GTK_ENTRY(ead_ead), str);


}


void
on_best_mix_calculate_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *best_mix_depth;
  GtkWidget *best_mix_ppo2_max;
  GtkWidget *best_mix_eanx_best;

  GtkWidget *ead_depth;
  GtkWidget *ead_ead;
  GtkWidget *ead_eanx;

  GtkWidget *mod_eanx;
  GtkWidget *mod_mod;
  GtkWidget *mod_ppo2_max;
  
  GtkWidget *blend_eanx_to;

  gfloat depth;
  gfloat ppo2_max;
  gfloat eanx_best;
  gchar str[128];

  gfloat ead;
  gfloat eanx;
  gfloat mod;

  best_mix_depth      = lookup_widget(GTK_WIDGET(button), "best_mix_depth");
  best_mix_ppo2_max   = lookup_widget(GTK_WIDGET(button), "best_mix_ppo2_max");
  best_mix_eanx_best = lookup_widget(GTK_WIDGET(button), "best_mix_eanx_best");

  depth    = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(best_mix_depth));
  ppo2_max = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(best_mix_ppo2_max));

  calculate_best_mix( ppo2_max, depth, &eanx_best);

  sprintf(str,"%.2f", eanx_best);
  gtk_entry_set_text(GTK_ENTRY(best_mix_eanx_best), str);


  // set blend
  blend_eanx_to        = lookup_widget(GTK_WIDGET(button), "blend_eanx_to");
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(blend_eanx_to), eanx_best);
  on_blend_calculate_clicked (button, NULL);

  // calculate MOD for this mix
  mod_eanx     = lookup_widget(GTK_WIDGET(button), "mod_eanx");
  mod_mod      = lookup_widget(GTK_WIDGET(button), "mod_mod");
  mod_ppo2_max = lookup_widget(GTK_WIDGET(button), "mod_ppo2_max");

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mod_eanx), eanx_best);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mod_ppo2_max), ppo2_max);

  calculate_mod(eanx_best, ppo2_max, &mod);
 
  sprintf(str,"%.2f", mod);
  gtk_entry_set_text(GTK_ENTRY(mod_mod), str);


  // calculate EAD for this mix
  ead_depth      = lookup_widget(GTK_WIDGET(button), "ead_depth");
  ead_ead        = lookup_widget(GTK_WIDGET(button), "ead_ead");
  ead_eanx       = lookup_widget(GTK_WIDGET(button), "ead_eanx");

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ead_eanx), eanx_best);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ead_depth), depth);
  
  calculate_ead(eanx_best, depth, &ead);
  sprintf(str,"%.2f", ead);
  gtk_entry_set_text(GTK_ENTRY(ead_ead), str);

  



}


void
on_ead_calculate_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *ead_depth;
  GtkWidget *ead_ead;
  GtkWidget *ead_eanx;

  GtkWidget *mod_eanx;
  GtkWidget *mod_mod;
  GtkWidget *mod_ppo2_max;
 
  gfloat mod, ppo2_max;

  gfloat depth;
  gfloat ead;
  gfloat eanx;
  gchar str[128];

  ead_depth      = lookup_widget(GTK_WIDGET(button), "ead_depth");
  ead_ead        = lookup_widget(GTK_WIDGET(button), "ead_ead");
  ead_eanx       = lookup_widget(GTK_WIDGET(button), "ead_eanx");

  depth    = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ead_depth));
  eanx     = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ead_eanx));

  calculate_ead(eanx, depth, &ead);

  sprintf(str,"%.2f", ead);
  gtk_entry_set_text(GTK_ENTRY(ead_ead), str);

  // calculate mod 
  mod_eanx     = lookup_widget(GTK_WIDGET(button), "mod_eanx");
  mod_mod      = lookup_widget(GTK_WIDGET(button), "mod_mod");
  mod_ppo2_max = lookup_widget(GTK_WIDGET(button), "mod_ppo2_max");

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mod_eanx), eanx);
  ppo2_max = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(mod_ppo2_max));
  calculate_mod(eanx, ppo2_max, &mod);
 
  sprintf(str,"%.2f", mod);
  gtk_entry_set_text(GTK_ENTRY(mod_mod), str);

  // FIXME:
  // insert warning: if depth > mod -> MOD exceeded for this mix @ ppO2!

}


void
on_mod_calculate_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *mod_eanx;
  GtkWidget *mod_mod;
  GtkWidget *mod_ppo2_max;
 
  gfloat eanx, mod, ppo2_max;
  gchar str[128];

  mod_eanx     = lookup_widget(GTK_WIDGET(button), "mod_eanx");
  mod_mod      = lookup_widget(GTK_WIDGET(button), "mod_mod");
  mod_ppo2_max = lookup_widget(GTK_WIDGET(button), "mod_ppo2_max");

  eanx     = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(mod_eanx));
  ppo2_max = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(mod_ppo2_max));

  calculate_mod (eanx, ppo2_max, &mod);

  sprintf(str,"%.2f", mod);
  gtk_entry_set_text(GTK_ENTRY(mod_mod), str);

}




void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *open;

  open = create_file_load_selection ();
  gtk_widget_show (open);
  file_load_mode = TRUE;

}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
show_error_window("Sorry!\nFile saving is not implemented yet.\n");

}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 
  show_error_window("Sorry!\nFile saving is not implemented yet.\n");

#if 0
  GtkWidget *save;

  save = create_file_load_selection ();
  gtk_widget_show (save);
  file_load_mode = FALSE;
#endif
}


void
on_file_select_ok_button1_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *file_selector;
  GtkWidget *button_clear;
  GtkWidget *spin;
  GtkWidget *select_dive;  

  GtkListStore *store;
  GtkTreeIter iter;

  GtkTreeModel *model;
 
  GtkWidget *scrolledwindow5;

  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreePath *path = NULL;

  int i;
  int parse_result;
  char date_str[80];

  struct udcf_profile_struct* udcf_ptr;
  struct udcf_group_struct* group_ptr;
  struct udcf_dive_struct* dive_ptr;

  enum
    {
      COLUMN_NUMBER,
      COLUMN_DATE,
      NUM_COLUMNS
    };


  spin  = lookup_widget(GTK_WIDGET(main_window), "spinbutton_delta_time");

  file_selector = gtk_widget_get_toplevel (GTK_WIDGET (button));
  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));

  DPRINTF("selected file: %s \n", selected_filename);

  if (file_load_mode) {
    if (file_loaded){
      // free stuff !
      udcf_free_data(udcf);
    } 
    /* load file, parse file, select dive and set up my_dive */
      
    DPRINTF("init data...\n");
    udcf = malloc(sizeof(struct udcf_profile_struct));
    udcf->version = 815;
    udcf->measure = 0;
    udcf->vendor =  "testvendor";
    udcf->model = "testmodel";
    udcf->driver_version = 0;
    udcf->personal_info = "mattzz";
    udcf->total_groups = 0;
    udcf->total_dives  = 0;
    udcf->serial_id    = "4711.0";
    udcf->group_start = NULL;
    udcf->group_end = NULL;
    
    init_dive_data(&my_dive);

    /* parse xml file */
    DPRINTF("parsing...\n");
    parse_result = udcf_parse_doc( selected_filename, udcf);
    
    switch(parse_result){
    case 1: // xml syntax error
      show_error_window("Error!\nXML syntax error!\n");
      return;
      break;
    case 2: // empty document
      show_error_window("Error!\nEmpty document!\n");
      return;
      break;
    case 3: // No <Profile> found
      show_error_window("Error!\nNo <PROFILE> node found.\n");
      return;
      break;
    }


    file_loaded = TRUE;

    if (udcf->total_dives == 0){
      DPRINTF("No dive data found!\n");
      show_error_window("Error!\n\nNo valid dive data found in file!");
    } else {
      DPRINTF("Found %d dive(s). Please select one.\n", udcf->total_dives);

      select_dive = create_select_dive_dialog ();
	  
      scrolledwindow5 = lookup_widget(GTK_WIDGET(select_dive), "scrolledwindow5");
	  
      /* create tree model */
      store = gtk_list_store_new (NUM_COLUMNS,
				  G_TYPE_UINT,
				  G_TYPE_STRING);
      

      /* adding data to list */

      /* traversing udcf struct for all dives in all groups */
      DPRINTF("traversing dives...\n");
      udcf_ptr = udcf;

      for(group_ptr = udcf->group_start; group_ptr; group_ptr = group_ptr->next_group){

	DPRINTF(" group %d...\n", group_ptr->group_number);
	for(dive_ptr=group_ptr->dive_start; dive_ptr; dive_ptr= dive_ptr->next_dive){
	  DPRINTF("  dive %d...\n", dive_ptr->dive_number);
	      
	      
	  sprintf(date_str, "%02d-%02d-%02d, %02d:%02d", 
		  dive_ptr->year,
		  dive_ptr->month,
		  dive_ptr->day,
		  dive_ptr->hour,
		  dive_ptr->minute);
		      
	  gtk_list_store_append (store, &iter);
	  gtk_list_store_set (store, &iter,
			      COLUMN_NUMBER, dive_ptr->dive_number,
			      COLUMN_DATE, date_str,
			      -1);
	}
      }
      DPRINTF("traversing done...\n");

      model = GTK_TREE_MODEL (store);

      /* create tree view */
      treeview = gtk_tree_view_new_with_model (model);

      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
      gtk_tree_view_set_search_column (GTK_TREE_VIEW (treeview),
				       COLUMN_DATE);


      /* select last row */
      path = gtk_tree_model_get_path (model, &iter);
      gtk_tree_view_set_cursor(GTK_TREE_VIEW (treeview), path, NULL, FALSE);

      g_object_unref (model);
      gtk_container_add (GTK_CONTAINER (scrolledwindow5), treeview);

      /* add column for dive number */
      renderer = gtk_cell_renderer_text_new ();
      column = gtk_tree_view_column_new_with_attributes ("Dive number",
							 renderer,
							 "text",
							 COLUMN_NUMBER,
							 NULL);
      //gtk_tree_view_column_set_sort_column_id (column, COLUMN_NUMBER);
      gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
      
      /* add column for dive text */
      renderer = gtk_cell_renderer_text_new ();
      column = gtk_tree_view_column_new_with_attributes ("Dive Date",
							 renderer,
							 "text",
							 COLUMN_DATE,
							 NULL);
      
      //gtk_tree_view_column_set_sort_column_id (column, COLUMN_DATE);
      gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
      
      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
      gtk_tree_view_set_search_column (GTK_TREE_VIEW (treeview),
				       COLUMN_DATE);
      
      //	  g_object_unref (model);
      gtk_widget_show_all (select_dive);

    }  
  } else {
    DPRINTF("saving...\n");
    /* save file */
  }
}


void
on_file_load_cancel_button1_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}







void
on_select_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_select_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkTreeIter iter;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(treeview));
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
  GtkWidget *spin;

  struct udcf_profile_struct* udcf_ptr;
  struct udcf_group_struct* group_ptr;
  struct udcf_dive_struct* dive_ptr;
  struct udcf_sample_struct* sample_ptr;
  
  spin  = lookup_widget(GTK_WIDGET(main_window), "spinbutton_delta_time");

  /* get number of selected dive */
  if (gtk_tree_selection_get_selected (selection, NULL, &iter))
    {
      gint i;
      GtkTreePath *path;

      path = gtk_tree_model_get_path (model, &iter);
      i = gtk_tree_path_get_indices (path)[0] + 1;
      udcf->selected_dive = i;
      DPRINTF("selected: %d\n", i);
      gtk_tree_path_free (path);


      /* fill my_dive with udcf dive data */

      udcf_ptr = udcf;

      for(group_ptr = udcf->group_start; group_ptr; group_ptr = group_ptr->next_group){
	DPRINTF(" group %d...\n", group_ptr->group_number);
	for(dive_ptr=group_ptr->dive_start; dive_ptr; dive_ptr= dive_ptr->next_dive){
	  DPRINTF("  dive %d...\n", dive_ptr->dive_number);
	  if (dive_ptr->dive_number == udcf->selected_dive){
	    my_dive.delta_time = dive_ptr->delta_time;
	    gtk_spin_button_set_value (GTK_SPIN_BUTTON(spin), my_dive.delta_time );
	    DPRINTF("   getting samples...\n");
	    i = 0;
	    for(sample_ptr = dive_ptr->samples_start; sample_ptr; sample_ptr=sample_ptr->next_sample){
	      //  DPRINTF("    sample: %2.2f\n", sample_ptr->depth);
	      my_dive.depth = sample_ptr->depth;
	      calculate_tissues(&my_dive);
	      i++;
	      my_dive.num_file_samples = i;
 
	    }
	  }
	}
      }
      DPRINTF("read %d samples\n", i);

      update_values(main_window);
      gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
    }
}



void show_error_window(char* error_string)
{
  GtkWidget *error_text;
  GtkWidget *error_dialog;

  GtkTextBuffer *error_text_buffer;
  GtkTextIter txtstart, txtend;
  //PangoFontDescription *font_desc;
 
  error_dialog = create_error_dialog ();
  gtk_widget_show (error_dialog);


  error_text = lookup_widget(GTK_WIDGET(error_dialog), "error_textview");

  error_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (error_text));
  //font_desc = pango_font_description_from_string ("Courier 10 Pitch");
  //gtk_widget_modify_font (error_text, font_desc);
  //pango_font_description_free (font_desc);
  gtk_text_buffer_get_bounds (error_text_buffer, &txtstart, &txtend); 
  gtk_text_buffer_delete (error_text_buffer, &txtstart, &txtend);
  //  sprintf(str,"testAdd Oxygen to %.2f,\ntop with pure air to %.2f", start_pressure + oxygen, start_pressure + oxygen + topping);

  gtk_text_buffer_insert_at_cursor (error_text_buffer, error_string, -1);

}

void
on_error_okbutton1_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}











