#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include "interface.h"
#include "support.h"
#include "plot.h"
#include "buehlmann.h"

extern dive_data my_dive;


void plot_dive (GdkPixmap *pixmap, GtkWidget *widget){
  GtkWidget *drawing_area;
  GdkRectangle  update_rect;
  GdkGC *gc_grid = gdk_gc_new(pixmap);
  GdkGC *gc_depth_min = gdk_gc_new(pixmap);
  GdkColor color;
  GdkFont *font; 
  
  gchar txt[10];
  gint i;
  gfloat j;
  gfloat grid_y, grid_x;
  gfloat dx, dy;


  /* line style grid */
  /* b | (g << 8) | (r << 16); we use a light grey */
  color.pixel = 128 | ( 128 << 8) | (128 << 16);
  gdk_gc_set_foreground(gc_grid, &color);
  gdk_gc_set_line_attributes(gc_grid, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_BEVEL);

  /*line style min tol depth */
  color.pixel = 0 | ( 0 << 8) | (128 << 16);
  gdk_gc_set_foreground(gc_depth_min, &color);
  gdk_gc_set_line_attributes(gc_depth_min, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_BEVEL);

  font = gdk_font_load("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*"); 
  drawing_area = lookup_widget(GTK_WIDGET(widget), "drawingarea1");

  /* clear drawing area */
  gdk_draw_rectangle (pixmap,
              drawing_area->style->white_gc,
              TRUE,
              0, 0,
              drawing_area->allocation.width,
              drawing_area->allocation.height);

  /* plot y axis */
  gdk_draw_line (pixmap, drawing_area->style->black_gc,
		 0, 0,
		 0, drawing_area->allocation.height);

  dx =   (drawing_area->allocation.width / (my_dive.num_samples + 1.0));
  dy =   (drawing_area->allocation.height / (my_dive.depth_max + 1.0));

  /* plotting all samples */
  for (i=0; i<my_dive.num_samples;i++){
    gdk_draw_line(pixmap, drawing_area->style->black_gc,
		  (i-1)*dx, my_dive.depth_samples[i-1]*dy,
		   i*dx, my_dive.depth_samples[i]*dy);
  }

  /* plotting depth_tol_min */
  for (i=0; i<my_dive.num_samples;i++){
    gdk_draw_line(pixmap, gc_depth_min,
		  (i-1)*dx, my_dive.depth_tol_min_samples[i-1]*dy,
		   i*dx, my_dive.depth_tol_min_samples[i]*dy);
  }


  /* plot grid */
  if(my_dive.depth_max > 50){
    grid_y = 10.0;
  } else {
    grid_y = 5.0;
  }
  if(my_dive.time > 3600){
    grid_x = 10.0;
  } else {
    grid_x = 5.0;
  }
  /* ygrid */
  for (j=0; j< drawing_area->allocation.height; j=j+dy*grid_y){
    gdk_draw_line(pixmap, gc_grid,
		  0, j,
		  drawing_area->allocation.width, j);

    sprintf(txt, "%3d", (int) rint(j/dy));
    gdk_draw_text (pixmap,
		   font,
                   drawing_area->style->black_gc,
		   0, j, 
		   txt, 3 );

  }

  /* xgrid */
  for (j=0; j< drawing_area->allocation.width;  j=j+(dx*grid_x)*(60.0/my_dive.delta_time)){
    //for (i=0; i< drawing_area->allocation.width; i=i+dx*grid_x){
    gdk_draw_line(pixmap, gc_grid,
		  j, 0,
		  j, drawing_area->allocation.height);

    sprintf(txt, "%3d", (int) rint((j*(my_dive.delta_time/60.0))/(dx))); // time 
    gdk_draw_text (pixmap,
		   font,
                   drawing_area->style->black_gc,
		   j, drawing_area->allocation.height , 
		   txt, 3 );


  }

	 

  /* --- Set the rect up
   * --- The entire pixmap is going to be copied
   * --- onto the window so the rect is configured
   * --- as the size of the window.
   */
  update_rect.x = 0;
  update_rect.y = 0;
  update_rect.width = drawing_area->allocation.width;
  update_rect.height = drawing_area->allocation.height;
  
  /* --- And then draw it (calls expose_event) --- */
  gtk_widget_draw (drawing_area, &update_rect);
  gdk_gc_unref(gc_grid);

}
