#ifndef UDCFPARSE_H
#define UDCFPARSE_H


#include <libxml/tree.h>
#include <libxml/parser.h>


typedef struct udcf_sample_struct* udcf_sample;
typedef struct udcf_dive_struct* udcf_dive;
typedef struct udcf_group_struct* udcf_group;
typedef struct udcf_profile_struct* udcf_profile;


/* ---- Flag for metric/imperial units ---- */
enum measure_enum{metric,imperial};
typedef enum measure_enum udcf_measure;

/* ---- A structure holding a mix of He, O2 and N2 ---- */
struct udcf_mix_struct {
  char* mix_name;
  double pressure_start, pressure_end;
  double tank_volume;
  double he_ratio;
  double o2_ratio;
  double n2_ratio;
  double acv;
};
typedef struct udcf_mix_struct udcf_mix;


/* ---- Datatype for sample-mode ---- */
enum sample_mode_enum{M_DELTA, M_TD};
typedef enum sample_mode_enum udcf_mode_type;

/* ---- Datatype for a point in XY-Mode ---- */
struct xy_point_struct {
  double x;
  double y;
};
typedef struct xy_point_struct udcf_point;

/* ---- Datatype for work-load ---- */
typedef double udcf_work;

/* ---- Datatyps for holding a sample ---- */
enum sample_type_enum{ST_UNKNOWN = 0, 
                      ST_DEPTH, 
                      ST_TD, 
                      ST_MARK, 
                      ST_MIX, 
                      ST_SAMPLETIME, 
                      ST_TEMPERATURE, 
                      ST_WORK};
typedef enum sample_type_enum udcf_sample_type;

/* ---- Special markers for Slow, Deco, Out-of-Breath  and Surface-Mode ---- */
enum marker_type_enum{C_UNKNOWN, 
		      C_ASC, 
		      C_DECO, 
		      C_SURF, 
		      C_BREATH, 
		      C_RBT, 
		      C_ERROR};
typedef enum marker_type_enum udcf_marker_type;

union udcf_data_union {
  double depth;
  double temperature;
  double sample_time;
  udcf_marker_type mark;
  udcf_point point;
  udcf_work work;
  long mix;
};
typedef union udcf_data_union udcf_sample_data;


struct udcf_sample_struct {
  struct udcf_sample_struct *next_sample;
  udcf_sample_type type;
  //  udcf_sample_data data;
  double depth;
} ;



struct udcf_dive_struct {
  struct udcf_dive_struct  *next_dive;
  int dive_number;
  int day, month, year;
  int  minute, hour;
  double surface_interval;
  char* place;
  double density;
  double temperature;
  double altitude;
  udcf_mix *mix_list;
  long mix_size;
  long mix_index;
  int delta_time;
  char *prg_data;
  udcf_mode_type mode;
  struct udcf_sample_struct *samples_start;
  struct udcf_sample_struct *samples_end;
  long user_data_type; /* Application specific data */
  void *user_data;    /*     "          "       "  */ 
};




struct udcf_group_struct { 
  struct udcf_group_struct* next_group;
  struct udcf_dive_struct* dive_start;
  struct udcf_dive_struct* dive_end;
  int group_number;
  int group_dives;
};


struct udcf_profile_struct {
  long version;
  struct udcf_measure_struct* measure;
  char* vendor;
  char* model;
  long driver_version;
  char* personal_info;
  long total_groups;
  long total_dives;
  long selected_dive;
  char* serial_id;
  struct udcf_group_struct* group_start;
  struct udcf_group_struct* group_end;
};



/* function prototypes */

int udcf_parse_doc( const char *filename, struct udcf_profile_struct* udcf);
void udcf_parse_repgroup (xmlDocPtr, xmlNodePtr, struct udcf_profile_struct* udcf);
void udcf_parse_dive (xmlDocPtr, xmlNodePtr, struct udcf_profile_struct* udcf);
void udcf_parse_dive_date (xmlDocPtr, xmlNodePtr, struct udcf_profile_struct* udcf);
void udcf_parse_dive_time (xmlDocPtr, xmlNodePtr, struct udcf_profile_struct* udcf);
void udcf_parse_samples (xmlDocPtr, xmlNodePtr, struct udcf_profile_struct* udcf);
void udcf_free_data(struct udcf_profile_struct* udcf);

#endif /* UDCFPARSE_H */
